#include "JRCSVFileReaderObj.h"
#include "JInteger.h"
#include "JIntegerData.h"
#include "JRealData.h"
#include "JStringData.h"
#include "JIntegerProperty.h"
#include "JIntegerListProperty.h"
#include "JStringProperty.h"
#include "JLinkObj.h"
#include "JBytesData.h"
#include <malloc.h>
#include <string.h>
static char* newstrcpy( char* buf ) {
	char* result = (char*)malloc( strlen( buf ) + 1 );
	strcpy( result, buf );
	return( result );
}
char* theJRCSVFileReaderObj = JRCSVFileReaderObj().Register();


void JRCSVFileReaderObj::writeContent(JOutputStream& os) {
  JAddObj::writeContent(os);
  putObject(os, "file", file);
}

void JRCSVFileReaderObj::readContent(JDictionary& dict) {
  JAddObj::readContent(dict);
  JObject* obj_file = getObject( dict, "file");
  if ( obj_file ) file = *(JString*)obj_file;
}

const char* JRCSVFileReaderObj::className() const { return "JRCSVFileReaderObj";}


JObject* JRCSVFileReaderObj::clone() const { return new JRCSVFileReaderObj(*this);}


// class constructor
JRCSVFileReaderObj::JRCSVFileReaderObj() {
  igm = 1 + 1 + 1;
  ogm = 1 + MAX_OUTPUTS;
  on = 0;
  tick = 0;
  r0 = 0;
  for ( int i = 0; i < MAX_OUTPUTS; i++ ) {
	  rout[i] = 0;
  }
  inStreamAvailable = 0;
}


void JRCSVFileReaderObj::draw(JGraphics g, int x, int y, int w, int h) {
  JRect rect(x, y, w, h/2);
  JRect rect2(x, y + h/2, w, h/2);
	  drawText( g, file, rect );
  if ( file.length() > 0 ) {
	  g.setJColor( JColor::white );
  drawText(g, JString("Real CSV"), rect2);
  }
}


boolean JRCSVFileReaderObj::inputAllowed(int n)
{ return !inputSet(n).last();}
JString JRCSVFileReaderObj::inputTag(int n) {
  char* tag[] = {
    "on",
    "tick",
  "inputFile" };
  return( tag[n] );
}


JString JRCSVFileReaderObj::outputTag(int n) {
  char* tag[] = {
    "r0",
    "r1",
    "r2",
    "r3",
    "r4",
    "r5",
    "r6",
    "r7",
    "r8",
    "r9",
    "r10",
	"r11",
	"r12",
	"r13",
	"r14",
	"r15",
	"r16",
	"r17",
	"r18",
	"r19",
	"r20",
	"r21",
	"r22",
	"r23",
	"r24",
	"r25",
	"r26",
	"r27",
	"r28",
	"r29",
	"r30",
	"r31",
	"r32",
	"r33",
	"r34",
	"r35",
	"r36",
	"r37",
	"r38",
	"r39",
	"r40",
	"r41"

  };
  return( tag[n] );
}
int JRCSVFileReaderObj::outputType( int n ) {
  if ( n == 0 ) return JIntegerData::id;
  return JRealData::id;
}
int JRCSVFileReaderObj::inputType( int n ) {
  if ( n != 2) return JIntegerData::id;
  return JStringData::id;
}

JFRect JRCSVFileReaderObj::inputArea(int n) {
  return topArea( n, 0, 1 + 1 + 1, 0.25 );
}


JFRect JRCSVFileReaderObj::outputArea(int n) {
  return rightArea(n, 0, 1 + MAX_OUTPUTS);
}


int JRCSVFileReaderObj::inputFace(int n) {
  return TOP;
}


int JRCSVFileReaderObj::outputFace(int n) {
  return RIGHT;
}


JFPoint JRCSVFileReaderObj::inputPoint(int n, JLinkObj& link) {
  return topPoint(n, link, 0, 1 + 1 +1);
}


JFPoint JRCSVFileReaderObj::outputPoint(int n, JLinkObj& link) {
  return rightPoint(n, link, 0, 1 + MAX_OUTPUTS);
}


void JRCSVFileReaderObj::access(int n, JLinkObj& link, const JDataType& data) {
  if ( n == 0 ) {
	INT(data) = r0;
  } else {
    DOUBLE(data) = rout[n-1];
  }
}

#include <ctype.h>
char dbgbuf[200];
char* grabDouble( char* scanner, double* xresult ) {
	if ( scanner == NULL ) {
		return( NULL );
	}
	//JComponent::debug( scanner );
	double result = 0;
	int fraction = 0;
	int fractionFactor = 1;
	bool mainResult = true;
	while ( *scanner ) {
		if ( isdigit( *scanner )) {
			if ( mainResult ) {
				result = result*10 + (*scanner - '0');
				sprintf( dbgbuf, "result now %e", result );
				//JComponent::debug( newstrcpy( dbgbuf ));
			} else {
				fraction = fraction * 10 + (*scanner - '0');
				fractionFactor *= 10;
				sprintf( dbgbuf, "fraction part now %d", fraction );
				//JComponent::debug( newstrcpy( dbgbuf ));
			}
		} else if ( *scanner == '.' ) {
			mainResult = false;
		} else {
			scanner++;
			*xresult = result + (double)fraction/(double)fractionFactor;
			sprintf( dbgbuf, "double value is %e", *xresult );
			//JComponent::debug( newstrcpy( dbgbuf ));
			return( scanner );
		}
		scanner++;
	}
	return( NULL );
}
char* grabInt( char* scanner, int* xresult ) {
	int result = 0;
	if ( scanner == NULL ) {
		return( NULL );
	}
	//JComponent::debug( scanner );
	while ( *scanner ) {
		if ( isdigit( *scanner )) {
			result = result*10 + (*scanner - '0');
		} else {
			scanner++;
			*xresult = result;
			return( scanner );
		}
		scanner++;
	}
	return( NULL );
}

void JRCSVFileReaderObj::engine(int n, JLinkObj& link) {
  switch (n) {
  case ON:
    link.access(JIntegerData(on));
	if ( on ) {
		inFile = JFile( file );
		if ( inFile.exists() ) {
			//JComponent::debug( "File exists" );
			inputStream = fopen( file, "r" );
			if ( inputStream == null ) {
				inStreamAvailable = 0;
			} else {
				inStreamAvailable = 1;
			}
		} else {
			//JComponent::debug( "File does not exist" );
			inStreamAvailable = 0;
		}
	} else {
	}
    break;
  case TICK:
    link.access(JIntegerData(tick));
	if ( tick ) {
		if ( inStreamAvailable ) {
			char buf[500];
			if ( fgets( &buf[0], 499, inputStream ) == NULL ) {
				inStreamAvailable = 0;
				fclose( inputStream );
				//JComponent::debug( "got to end of file" );
			} else {
				char* scanner = &buf[0];
				scanner = grabInt( &buf[0], &r0 );
				if ( scanner ) broadcast( 0 );
				for ( int i = 0; i < MAX_OUTPUTS; i++ ) {
					scanner = grabDouble( scanner, &rout[i] );
					if ( scanner ) broadcast( i+1 );
					else break;
				}
			}

			//JString x;
			//inputStream >> x;
			//JComponent::debug( "test" );
		}
	}
    break;
   case INFILE:
	   link.access(JStringData( file ));
	   break;

   default:
    break;
  }
}


JArray JRCSVFileReaderObj::getProperties() {
  JArray properties;
  properties.append(JStringProperty("file", file));
  return properties;
}


boolean JRCSVFileReaderObj::updateProperty(JProperty& prop) {
  if (prop.getName() == JString("file")) {
    file = ((JStringProperty*)&prop)->value;
    return true;
  }
  return false;
}


