#include "JWordMakerObj.h"
#include "JInteger.h"
#include "JIntegerData.h"
#include "JStringData.h"
#include "JIntegerProperty.h"
#include "JIntegerListProperty.h"
#include "JStringProperty.h"
#include "JLinkObj.h"
#include "JBytesData.h"


char* theJWordMakerObj = JWordMakerObj().Register();

const char* JWordMakerObj::className() const { return "JWordMakerObj";}


JObject* JWordMakerObj::clone() const { return new JWordMakerObj(*this);}


// class constructor
JWordMakerObj::JWordMakerObj() {
  igm = 2;
  ogm = 1;
  internalWordIdx = 0;
  include_capital = true;
  include_lowercase = true;
  include_digits = true;
  include_printable = false;
}


void JWordMakerObj::draw(JGraphics g, int x, int y, int w, int h) {
  JRect rect(x, y, w, h);
  drawText(g, JString("mkWord"), rect);
}


boolean JWordMakerObj::inputAllowed(int n)
{ return !inputSet(n).last();}
JString JWordMakerObj::inputTag(int n) {
  char* tag[] = {
    "letter", "backspace" };
  return( tag[n] );
}


int JWordMakerObj::outputType( int n ) {
  if ( n == 0 ) return JStringData::id;
  return JIntegerData::id;
}
int JWordMakerObj::inputType( int n ) {
  if ( n == 0 ) return JStringData::id;
  return JIntegerData::id;
}
JString JWordMakerObj::outputTag(int n) {
  char* tag[] = {
    "word" };
  return( tag[n] );
}


JFRect JWordMakerObj::inputArea(int n) {
  //if ( n < ( 1 )) {
    return leftArea( n, 0, 2, 0.1, 0.8 );
  //}
}


JFRect JWordMakerObj::outputArea(int n) {
  return rightArea(n, 0, 1);
}


int JWordMakerObj::inputFace(int n) {
  if ( n < 2 ) {
    return LEFT;
  } else {
    return TOP;
  }
}


int JWordMakerObj::outputFace(int n) {
  return RIGHT;
}


JFPoint JWordMakerObj::inputPoint(int n, JLinkObj& link) {
  return leftPoint( n, link, 0, 2, 0.1, 0.8);
}


JFPoint JWordMakerObj::outputPoint(int n, JLinkObj& link) {
  return rightPoint(n, link, 0, 1);
}


void JWordMakerObj::access(int n, JLinkObj& link, const JDataType& data) {
  if ( n == 0 ) {
    JSTRING(data) = word;
  }
}

#include <ctype.h>
#include <stdio.h>
#include <malloc.h>
#include <string.h>
char* newstrcpy( char* s ) {
	char* result = (char*)malloc( strlen( s ) + 1 );
	strcpy( result, s );
	return result;
}
bool JWordMakerObj::isdelimiter( char s ) {
	if ( isspace( s )) {
		return( true );
	} else if ( include_printable ) {
		return( !isprint( s ));
	} else {
		int charIsUpper = isupper( s );
		if ( charIsUpper && include_capital ) {
			return( false );
		}
		int charIsLower = islower( s );
		if ( charIsLower && include_lowercase ) {
			return( false );
		}
		int charIsDigit = isdigit( s );
		if ( charIsDigit && include_digits ) {
			return( false );
		}
		return( true );
	}
}
void JWordMakerObj::processText( char* s ) {
	while ( *s ) {
		if ( isdelimiter( *s ) || ( internalWordIdx == 100 )) {
			if ( internalWordIdx > 0 ) {
				word = JString( internalWord, 2 );
				broadcast( 0 );
				internalWordIdx = 0;
				internalWord[0] = 0;
			}
		} else {
			internalWord[ internalWordIdx++ ] = *s;
			internalWord[ internalWordIdx ] = 0;
		}
		s++;
	}
}

JArray JWordMakerObj::getProperties() {
	JArray properties = JAddObj::getProperties();
	properties.append( JIntegerListProperty( "include_capital", include_capital, JIntegerListProperty::booleanTag ));
	properties.append( JIntegerListProperty( "include_lowercase", include_lowercase, JIntegerListProperty::booleanTag ));
	properties.append( JIntegerListProperty( "include_digits", include_digits, JIntegerListProperty::booleanTag ));
	properties.append( JIntegerListProperty( "include_printable", include_printable, JIntegerListProperty::booleanTag ));
	return( properties );
}

void JWordMakerObj::writeContent( JOutputStream& os ) {
	JAddObj::writeContent( os );
	putInteger( os, "include_capital", include_capital );
	putInteger( os, "include_lowercase", include_lowercase );
	putInteger( os, "include_digits", include_digits );
	putInteger( os, "include_printable", include_printable );
}

void JWordMakerObj::readContent( JDictionary& dict ) {
	JAddObj::readContent( dict );
	include_capital = getInteger( dict, "include_capital" );
	include_lowercase = getInteger( dict, "include_lowercase" );
	include_digits = getInteger( dict, "include_digits" );
	include_printable = getInteger( dict, "include_printable" );
}

boolean JWordMakerObj::updateProperty( JProperty& prop ) {
	if ( prop.getName() == JString( "include_capital" )) {
		include_capital = ((JIntegerProperty*)&prop)->value;
		return true;
	} else if ( prop.getName() == JString( "include_lowercase" )) {
		include_lowercase = ((JIntegerProperty*)&prop)->value;
		return true;
	} else if ( prop.getName() == JString( "include_digits" )) {
		include_digits = ((JIntegerProperty*)&prop)->value;
		return true;
	} else if ( prop.getName() == JString( "include_printable" )) {
		include_printable = ((JIntegerProperty*)&prop)->value;
		return true;
	} else {
		return false;
	}
}

void JWordMakerObj::backspace() {
	//char dbuf[100];
	//sprintf( dbuf, "Before backspace: '%s'", internalWord );
	//JComponent::debug( newstrcpy( dbuf ));
	if ( internalWordIdx > 0 ) {
		internalWordIdx--;
		internalWord[ internalWordIdx ] = 0;
	}
	//sprintf( dbuf, "After backspace: '%s'", internalWord );
	//JComponent::debug( newstrcpy( dbuf ));
}

void JWordMakerObj::engine(int n, JLinkObj& link) {
  switch (n) {
  case LETTER:
    link.access( JStringData( letter ));
	processText( letter );
    break;
  case BACKSPACE:
	  link.access( JIntegerData(backspaceIndicator ));
	  if ( backspaceIndicator )
	  backspace();
	  break;
  default:
    break;
  }
}


