#include "JCALObj.h"
#include "JInteger.h"
#include "JIntegerData.h"
#include "JStringData.h"
#include "JIntegerProperty.h"
#include "JIntegerListProperty.h"
#include "JStringProperty.h"
#include "JLinkObj.h"
#include "JBytesData.h"


char* theJCALObj = JCALObj().Register();


const char* JCALObj::className() const { return "JCALObj";}


JObject* JCALObj::clone() const { return new JCALObj(*this);}


// class constructor
JCALObj::JCALObj() {
  igm = 1;
  ogm = 1;
  uncalibrated = 0;
  minVal = 0;
  maxVal = 0;
  count = 0;
}


void JCALObj::draw(JGraphics g, int x, int y, int w, int h) {
  JRect rect(x, y, w, h);
  drawText(g, JString("CAL"), rect);
}


boolean JCALObj::inputAllowed(int n)
{ return !inputSet(n).last();}
JString JCALObj::inputTag(int n) {
  char* tag[] = {
    "uncalibrated" };
  return( tag[n] );
}


JString JCALObj::outputTag(int n) {
  char* tag[] = {
    "calibrated" };
  return( tag[n] );
}


JFRect JCALObj::inputArea(int n) {
//  if ( n < ( 1 )) {
    return leftArea( n, 0, 1, 0.1, 0.8 );
//  }
}


JFRect JCALObj::outputArea(int n) {
  return rightArea(n, 0, 1);
}


int JCALObj::inputFace(int n) {
  if ( n < 1 ) {
    return LEFT;
  } else {
    return TOP;
  }
}


int JCALObj::outputFace(int n) {
  return RIGHT;
}


JFPoint JCALObj::inputPoint(int n, JLinkObj& link) {
  return leftPoint( n, link, 0, 1, 0.1, 0.8);
}


JFPoint JCALObj::outputPoint(int n, JLinkObj& link) {
  return rightPoint(n, link, 0, 1);
}


void JCALObj::access(int n, JLinkObj& link, const JDataType& data) {
  if ( n == 0 ) {
    INT(data) = calibrated;
  }
}


void JCALObj::engine(int n, JLinkObj& link) {
	int range = 0;
  switch (n) {
  case UNCALIBRATED:
    link.access(JIntegerData(uncalibrated));
	if (( minVal == maxVal ) && ( minVal == 0 )) {
		minVal = maxVal = uncalibrated;
	} else if ( uncalibrated < minVal ) {
		minVal = uncalibrated;
	} else if ( maxVal < uncalibrated ) {
		maxVal = uncalibrated;
	}
	range = maxVal - minVal;
	if ( range > 0 ) {
		calibrated = (uncalibrated - minVal)*65535/range;
		int temp = calibrated/100;
		if ( temp != lastOutputValue ) {
			count++;
			if (( count % 100 ) == 0 ) {
				lastOutputValue = temp;
				broadcast( 0 );
			}
		}
	}
    break;
  default:
    break;
  }
}


